
/*@******************************************************
 *      COSTANTS DEFINITION
 ********************************************************/
#define PORT    12866
#define BPORT   12900
#define NUM_CAR 1024
  
/*@******************************************************
 *      DATA STRUCTURES
 ********************************************************/
typedef struct DRT_s
{
  int  num_ifs;     //number of net ifaces
  char *ifname[3];  //vector of iface names
  int  numprocs;    //number of running processes
  char mroute_is_on;//Flag for mrouted monitoring
  char ring_status; //Status of the FDDI ring: 1=up, 0=down
}DRT_t;



struct cmd_t
{
  char  cmd;
  char  par_nr;
  char *params[10];
}cmd_struct;	


struct NTPStatus
{
	char sender_addr[16];
	char server_addr[16];
	char syncro_status;
	char sign;
	struct timeval delta;
}Nstatus;

/*@******************************************************
 *      GLOBAL VARIABLES DECLARATION SECTION
 ********************************************************/

enum commands
{
  DIAG=0, 
  IFCONFIG,/*1*/ 
  GETTIME, /*2*/ 
  SETSERVL,/*3*/ 
  SETSERVN,/*4*/ 
  SETCLIENT,/*5*/ 
  NTPSTART,/*6*/ 
  NTPCTRL, /*7*/ 
  NTPGETST,/*8*/ 
  NTPSTOP, /*9*/ 
  GENERIC, /*10*/
  DTIUINIT,/*11*/
  ROUTTBL, /*12*/
  NTPFLUSH,/*13*/ 
  SNMPCONF,/*14*/
  SNMPSTART,/*15*/
  SETTIME  /*16*/
};

enum syncstat
{
	ERROR=-1, 
	SYNCRO, 
	UNSYNCRO, 
	SYNCRO_W_ERR, 
	SYNCRO_LOST
};

struct routtbl {
  u_long addr;
  u_long rout;
};

